@@ -3,9 +3,9 @@
 ;; Copyright (C) 2017
 
 ;; Author: Wilfred Hughes <me@wilfred.me.uk>
-;; Version: 0.5
+;; Version: 0.7
 ;; Keywords: convenience
-;; Package-Requires: ((emacs "24.4") (loop "1.3") (dash "2.13.0") (s "1.11.0") (f "0.18.2"))
+;; Package-Requires: ((emacs "24.4") (loop "1.3") (dash "2.13.0") (s "1.11.0") (f "0.18.2") (spinner "1.7.3"))
 ;; URL: https://github.com/Wilfred/suggest.el
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -32,16 +32,36 @@
 (require 'loop)
 (require 's)
 (require 'f)
+(require 'spinner)
 (require 'subr-x)
-(eval-when-compile
-  (require 'cl-lib)) ;; cl-incf
+(require 'cl-extra) ;; cl-prettyprint
+(require 'cl-lib) ;; cl-incf, cl-evenp, cl-oddp
+
+(defcustom suggest-insert-example-on-start t
+  "If t, insert example data in suggest buffer, else don't."
+  :group 'suggest
+  :type 'boolean)
 
 ;; See also `cl--simple-funcs' and `cl--safe-funcs'.
 (defvar suggest-functions
   (list
    ;; TODO: add funcall, apply and map?
+   ;; Boolean functions
+   #'not
+   ;; Type predicates
+   #'arrayp
+   #'atom
+   #'booleanp
+   #'consp
+   #'floatp
+   #'functionp
+   #'integerp
+   #'listp
+   #'numberp
+   #'sequencep
+   #'stringp
+   #'symbolp
    ;; Built-in functions that access or examine lists.
-   ;; TODO: why isn't car marked as pure?
    #'car
    #'cdr
    #'cadr
@@ -49,6 +69,7 @@
    #'last
    #'cons
    #'nth
+   #'nthcdr
    #'list
    #'length
    #'safe-length
@@ -67,6 +88,7 @@
    #'cl-first
    #'cl-second
    #'cl-third
+   #'cl-list*
    ;; dash.el list functions.
    #'-non-nil
    #'-slice
@@ -102,6 +124,7 @@
    #'-intersection
    #'-distinct
    #'-rotate
+   #'-sort
    #'-repeat
    #'-cons*
    #'-snoc
@@ -111,10 +134,6 @@
    #'-first-item
    #'-last-item
    #'-butlast
-   ;; dash.el folding/unfolding
-   #'-reduce
-   #'-reduce-r
-   #'-iterate
    ;; alist functions
    #'assoc
    #'alist-get
@@ -128,6 +147,7 @@
    #'hash-table-values
    ;; vectors
    ;; TODO: there must be more worth using
+   #'vector
    #'vconcat
    ;; Arithmetic
    #'+
@@ -154,6 +174,10 @@
    #'ftruncate
    #'1+
    #'1-
+   #'cl-evenp
+   #'natnump
+   #'cl-oddp
+   #'zerop
    ;; Logical operators
    #'lsh
    #'logand
@@ -171,6 +195,12 @@
    #'capitalize
    #'replace-regexp-in-string
    #'format
+   #'string-join
+   #'string-prefix-p
+   #'string-suffix-p
+   #'string-remove-prefix
+   #'string-remove-suffix
+   #'prin1-to-string
    ;; Quoting strings
    #'shell-quote-argument
    #'regexp-quote
@@ -214,7 +244,6 @@
    #'symbol-value
    #'symbol-file
    #'intern
-   #'read
    ;; Converting between types
    #'string-to-list
    #'string-to-number
@@ -257,7 +286,16 @@
    #'ignore
    )
   "Functions that suggest will consider.
-These functions must not produce side effects.
+
+These functions must not produce side effects, and must not be
+higher order functions.
+
+Side effects are obviously bad: we don't want to call
+`delete-file' with arbitrary strings!
+
+Higher order functions are any functions that call `funcall' or
+`apply'. These are not safe to call with arbitrary symbols, but
+see `suggest-funcall-functions'.
 
 The best functions for examples generally take a small number of
 arguments, and no arguments are functions. For other functions,
@@ -268,21 +306,68 @@ need multiple examples to ensure they do what the user wants.
 
 See also `suggest-extra-args'.")
 
+(defvar suggest-funcall-functions
+  (list
+   ;; Higher order list functions.
+   #'mapcar
+   ;; When called with a symbol, read will call it.
+   #'read
+   ;; dash.el higher order list functions.
+   #'-map
+   #'-mapcat
+   ;; dash.el folding/unfolding
+   #'-reduce
+   #'-reduce-r
+   #'-iterate
+   )
+  "Pure functions that may call `funcall'. We will consider
+consider these, but only with arguments that are known to be safe." )
+
 (defvar suggest-extra-args
   (list
    ;; There's no special values for `list', and it produces silly
    ;; results when we add values.
    #'list '()
-   ;; Similarly, we can use nil with `car' to build a list, but
-   ;; otherwise we're just building an irrelevant list if we use the
-   ;; default values.
-   #'car '(nil)
+   ;; Similarly, we can use nil when building a list, but otherwise
+   ;; we're just building an irrelevant list if we use the default
+   ;; values.
+   #'cons '(nil)
+   #'-cons* '(nil)
+   #'-snoc '(nil)
+   #'append '(nil)
    ;; `format' has specific formatting strings that are worth trying.
    #'format '("%d" "%o" "%x" "%X" "%e" "%c" "%f" "%s" "%S")
    ;; `-iterate' is great for building incrementing/decrementing lists.
-   ;; (an alternative to `number-sequence'.
+   ;; (an alternative to `number-sequence').
    #'-iterate '(1+ 1-)
-   ;; These common values often set flags in interesting ways.
+   ;; Lists can be sorted in a variety of ways.
+   #'-sort '(< > string< string>)
+   ;; For indexing functions, just use non-negative numbers. This
+   ;; avoids confusing results like (last nil 5) => nil.
+   #'elt '(0 1 2)
+   #'nth '(0 1 2)
+   #'nthcdr '(0 1 2)
+   #'last '(0 1 2)
+   #'-drop '(0 1 2)
+   #'-drop-last '(0 1 2)
+   #'-take '(0 1 2)
+   ;; For functions that look up a value, don't supply any extra
+   ;; arguments.
+   #'plist-member '()
+   #'assoc '()
+   ;; Likewise for comparisons, there's no interesting extra value we can offer.
+   #'-is-suffix-p '()
+   #'-is-prefix-p '()
+   #'-is-infix-p '()
+   ;; Remove has some weird behaviours with t: (remove 'foo t) => t.
+   ;; Only use nil as an extra value, so we can discover remove as an
+   ;; alternative to `-non-nil'.
+   #'remove '(nil)
+   ;; mapcar with identity is a fun way to convert sequences (strings,
+   ;; vectors) to lists.
+   #'mapcar '(identity)
+   ;; These common values often set flags in interesting
+   ;; ways.
    t '(nil t -1 0 1 2))
   "Some functions work best with a special extra argument.
 
@@ -291,7 +376,12 @@ produce good results. If a function isn't explicitly mentioned,
 we look up `t' instead.")
 
 (defun suggest--safe (fn args)
-  "Is FN safe to call with ARGS?"
+  "Is FN safe to call with ARGS?
+
+Safety here means that we:
+
+* don't have any side effects, and
+* don't crash Emacs."
   (not
    (or
     ;; Due to Emacs bug #25684, string functions that call
@@ -304,7 +394,29 @@ we look up `t' instead.")
          (< (car args) 0))
     ;; If `read' is called with nil or t, it prompts interactively.
     (and (eq fn 'read)
-         (member args '(nil (nil) (t)))))))
+         (member args '(nil (nil) (t))))
+    ;; Work around https://github.com/Wilfred/suggest.el/issues/37 on
+    ;; Emacs 24, where it's possible to crash `read' with a list.
+    (and (eq fn 'read)
+         (eq emacs-major-version 24)
+         (consp (car args)))
+    ;; Work around https://github.com/magnars/dash.el/issues/241
+    (and (memq fn '(-interleave -zip))
+         (null args))
+    (and (memq fn suggest-funcall-functions) ;
+         ;; TODO: what about circular lists?
+         ;;
+         ;; Does apply even handle that nicely? It looks like apply
+         ;; tries to get the length of the list and hangs until C-g.
+         (format-proper-list-p args)
+         (--any (or
+                 ;; Don't call any higher order functions with symbols that
+                 ;; aren't known to be safe.
+                 (and (symbolp it) (not (memq it suggest-functions)))
+                 ;; Don't allow callable objects (interpreted or
+                 ;; byte-compiled function objects).
+                 (functionp it))
+                args)))))
 
 (defface suggest-heading
   '((((class color) (background light)) :foreground "DarkGoldenrod4" :weight bold)
@@ -335,7 +447,8 @@ we look up `t' instead.")
     ;; Start the overlay after the ";; " bit.
     (let ((overlay (make-overlay (+ 3 start) end)))
       ;; Highlight the text in the heading.
-      (overlay-put overlay 'face 'suggest-heading))))
+      (overlay-put overlay 'face 'suggest-heading)))
+  (insert "\n"))
 
 (defun suggest--on-heading-p ()
   "Return t if point is on a heading."
@@ -374,7 +487,11 @@ we look up `t' instead.")
 
 (defun suggest--keybinding (command keymap)
   "Find the keybinding for COMMAND in KEYMAP."
-  (car (where-is-internal command keymap)))
+  (where-is-internal command keymap t))
+
+(defun suggest--insert-section-break ()
+  "Insert section break."
+  (insert "\n\n"))
 
 ;;;###autoload
 (defun suggest ()
@@ -389,13 +506,22 @@ and outputs given."
     (suggest-mode)
 
     (suggest--insert-heading suggest--inputs-heading)
-    (insert "\n1\n2\n\n")
+    (when suggest-insert-example-on-start
+      (insert "1\n2"))
+
+    (suggest--insert-section-break)
+
     (suggest--insert-heading suggest--outputs-heading)
-    (insert "\n3\n\n")
+    (when suggest-insert-example-on-start
+      (insert "3"))
+
+    (suggest--insert-section-break)
+
     (suggest--insert-heading suggest--results-heading)
-    (insert "\n")
     ;; Populate the suggestions for 1, 2 => 3
-    (suggest-update)
+    (when suggest-insert-example-on-start
+      (suggest-update))
+
     ;; Put point on the first input.
     (suggest--nth-heading 1)
     (forward-line 1))
@@ -408,7 +534,7 @@ and outputs given."
 N counts from 1."
   (goto-char (point-min))
   (let ((headings-seen 0))
-    (loop-while (< headings-seen n)
+    (while (< headings-seen n)
       (when (suggest--on-heading-p)
         (cl-incf headings-seen))
       (forward-line 1)))
@@ -599,7 +725,7 @@ This is primarily for quoting symbols."
     ;; See if (func value1 value2...) gives us a value.
     (-when-let (result (suggest--call func input-values input-literals))
       (push result outputs))
-    
+
     ;; See if (apply func input-values) gives us a value.
     (when (and (eq (length input-values) 1) (listp (car input-values)))
       (-when-let (result (suggest--call func input-values input-literals t))
@@ -632,7 +758,13 @@ than their values."
         this-iteration
         intermediates
         (intermediates-count 0)
-        (value-occurrences (make-hash-table :test #'equal)))
+        (value-occurrences (make-hash-table :test #'equal))
+        (funcs (append suggest-functions suggest-funcall-functions))
+        ;; Since we incrementally build a huge list of results,
+        ;; garbage collection does a load of work but doesn't find
+        ;; much to collect. Disable GC, as it significantly improves
+        ;; performance.
+        (gc-cons-threshold 50000000))
     ;; Setup: no function calls, all permutations of our inputs.
     (setq this-iteration
           (-map (-lambda ((values . literals))
@@ -644,7 +776,10 @@ than their values."
     (catch 'done
       (dotimes (iteration suggest--search-depth)
         (catch 'done-iteration
-          (dolist (func suggest-functions)
+          (dolist (func funcs)
+            ;; We need to call redisplay so the spinner keeps rotating
+            ;; as we search.
+            (redisplay)
             (loop-for-each item this-iteration
               (let ((literals (plist-get item :literals))
                     (values (plist-get item :values))
@@ -664,7 +799,7 @@ than their values."
                        (cl-incf possibilities-count)
                        (when (>= possibilities-count suggest--max-possibilities)
                          (throw 'done nil))
-                       
+
                        ;; If we're on the first iteration, we're just
                        ;; searching all input permutations. Don't try any
                        ;; other permutations, or we end up showing e.g. both
@@ -744,10 +879,15 @@ than their values."
      (t
       (< (length (plist-get pos1 :funcs)) (length (plist-get pos2 :funcs)))))))
 
+(defvar suggest--spinner nil)
+
 ;;;###autoload
 (defun suggest-update ()
   "Update the suggestions according to the latest inputs/output given."
   (interactive)
+  (setq suggest--spinner (spinner-create 'progress-bar t))
+  (spinner-start suggest--spinner)
+
   ;; TODO: error on multiple inputs on one line.
   (let* ((raw-inputs (suggest--raw-inputs))
          (inputs (--map (suggest--read-eval it) raw-inputs))
@@ -759,7 +899,7 @@ than their values."
     (setq possibilities
           (-take 5
                  (-sort #'suggest--cmp-relevance possibilities)))
-    
+
     (if possibilities
         (suggest--write-suggestions
          possibilities
@@ -767,10 +907,12 @@ than their values."
          ;; users use variables, we show the value of that variable.
          desired-output)
       (suggest--write-suggestions-string ";; No matches found.")))
+  (setq suggest--spinner nil)
   (suggest--update-needed nil)
   (set-buffer-modified-p nil))
 
-(define-derived-mode suggest-mode emacs-lisp-mode "Suggest"
+(define-derived-mode suggest-mode emacs-lisp-mode
+  '("Suggest" (:eval (spinner-print suggest--spinner)))
   "A major mode for finding functions that provide the output requested.")
 
 (define-key suggest-mode-map (kbd "C-c C-c") #'suggest-update)
